@@ -18,29 +18,45 @@
 import os
 import os.path
 import sys
-import commands
 import rpmUtils
+try:
+    test = rpmUtils.transaction.initReadOnlyTransaction()
+except:
+    import rpmUtils.transaction
 import rpm
 import glob
 import shutil
 import types
 import grp
+import stat
+import time
 from exceptions import Exception
 
 from optparse import OptionParser
 
-__VERSION__ = '0.4'
+__VERSION__ = '0.6'
 
 def error(msg):
     print >> sys.stderr, msg
 
 
+# result/exit codes
+# 0 = yay!
+# 1 = something happened  - it's bad
+# 30 = Yum emitted an error of some sort
+# 40 = some error in the pkg we're building
+# 10 = problem building the package
+# 20 = error in the chroot of some kind
+
 class Error(Exception):
     def __init__(self, msg):
         Exception.__init__(self)
         self.msg = msg
         self.resultcode = 1
 
+    def __str__(self):
+        return self.msg
+
 class YumError(Error): 
     def __init__(self, msg):
         Error.__init__(self, msg)
@@ -66,21 +82,46 @@ class RootError(Error):
         self.resultcode = 20
 
 
+class LogBuffer:
+
+    lines = []
+
+    def clear(self):
+        self.lines = []
+
+    def write(self, line):
+        if line[-1] != '\n':
+            line += '\n'
+        self.lines.append(line)
+
+    def writelines(self, lines):
+        for l in lines:
+            self.write(l)
+
+    def readlines(self):
+        return self.lines
+
+    def flush(self):
+        pass
 
 class Root:
     """base root object"""
     def __init__(self, config):
         self._state = 'unstarted'
-        self.tmplog = []
+        self.tmplog = LogBuffer()
         self.config = config
         root = config['root']
         if config.has_key('unique-ext'):
             root = "%s-%s" % (root, config['unique-ext'])
         self.basedir = os.path.join(config['basedir'], root)
+        if self.basedir.find("/var/lib/mock") != 0:
+            raise RootError, "Cannot change basedir location!"
         self.target_arch = config['target_arch']
         self.rootdir = os.path.join(self.basedir, 'root')
         self.homedir = self.config['chroothome']
         self.builddir = os.path.join(self.homedir, 'build')
+        self.cache_file = os.path.join(self.config['basedir'], 
+                self.config['cache_topdir'], self.config['root'] + self.config['cache_ext'])
         if not self.config.has_key('resultdir'):
             self.resultdir = os.path.join(self.basedir, 'result')
         else:
@@ -115,38 +156,23 @@ class Root:
         cfgout.write('statedir = %s\n' % self.statedir)
         cfgout.flush()
         cfgout.close()
-
-    def build_log(self, content):
-        if type(content) is types.ListType:
-            for line in content:
-                self._build_log.write('%s\n' % line)
-        elif type(content) is types.StringType:
-            self._build_log.write('%s\n' % content)
-        else:
-            # wtf?
-            pass
-        self._build_log.flush()
+    
+    def log(self, msg):
+        if self.config['quiet']: return
+        print msg
 
     def root_log(self, content):
-            
+
+        if type(content) is list:
+            self.tmplog.writelines(content)
+        else:
+            self.tmplog.write(content)
+        
         # do this so if the log dir isn't ready yet we can still get those logs
-        self.tmplog.append(content)
-        
-        if not hasattr(self, '_root_log'):
-            return
-        
-        for content in self.tmplog:
-            if type(content) is types.ListType:
-                for line in content:
-                    self._root_log.write('%s\n' % line)
-            elif type(content) is types.StringType:
-                self._root_log.write('%s\n' % content)
-            else:
-                # wtf?
-                pass
-            
+        if hasattr(self, '_root_log'):
+            self._root_log.writelines(self.tmplog.readlines())
             self._root_log.flush()
-        self.tmplog = [] # zero out the logs
+            self.tmplog.clear()
 
     def debug(self, msg):
         if self.config['debug']:
@@ -163,7 +189,7 @@ class Root:
             self._umount('dev/pts')
             
         if os.path.exists(self.basedir):
-            cmd = '%s -rfv %s' % (self.config['rm'], self.basedir)
+            cmd = '%s -rf %s' % (self.config['rm'], self.basedir)
             (retval, output) = self.do(cmd)
 
             if retval != 0:
@@ -183,38 +209,67 @@ class Root:
             sfo.write('%s\n' % curstate)
             sfo.close()
             self._state = curstate
-            print curstate
+            self.log(curstate)
         else:
             return self._state
+
+    def unpack(self):
+        self.state('unpack cache')
+        cmd = '%s %s %s' % (self.config['unpack_cmd'], self.basedir, self.cache_file)
+        self.do(cmd)
+
+    def pack(self):
+        self.state('create cache')
+        self._ensure_dir(os.path.join(self.config['basedir'], self.config['cache_topdir']))
+        cmd = '%s %s %s root' % (self.config['pack_cmd'], self.basedir, self.cache_file)
+        self.do(cmd)
     
     def prep(self):
         self.state("prep")
+        self.log("This may take a while")
 
-        self._prep_install()
-        if self.config['clean']:
-            cmd = 'groupinstall %s' % self.config['buildgroup']
+        create_cache=0
+        if self.config['use_cache']:
+            cache_exists = os.path.exists( self.cache_file )
+            if cache_exists:
+                cache_mtime = os.stat(self.cache_file)[stat.ST_MTIME]
+                cache_age_days = (time.time() - cache_mtime) / (60 * 60 * 24)
+                if cache_age_days > self.config['max_cache_age_days']:
+                    self.config["rebuild_cache"] = True
+        
+            if cache_exists and not self.config['rebuild_cache']:
+                if self.config['clean']:
+                    self.unpack()
+                cmd = 'update'
+            else:
+                cmd = '%s' % self.config['chroot_setup_cmd']
+                create_cache = 1
         else:
-            cmd = 'update'
-
+            if self.config['clean']:
+                cmd = '%s' % self.config['chroot_setup_cmd']
+            else:
+                cmd = 'update'
+         
+        self._prep_install()
         self.yum(cmd)
         self._prep_build()
-        
+         
+        if create_cache:
+            self.pack()
 
     def yum(self, cmd):
         """use yum to install packages/package groups into the chroot"""
-        # mach-helper yum --installroot=rootdir cmd
+        # mock-helper yum --installroot=rootdir cmd
         basecmd = '%s --installroot %s' % (self.config['yum'], self.rootdir)
         
         self._mount() # check it again        
         command = '%s %s' % (basecmd, cmd)
         self.debug("yum: command %s" % command)
 
-        self.root_log(command)
         (retval, output) = self.do(command)
-        self.root_log(output)
 
         if retval != 0:
-            raise YumError, "Error peforming yum command: %s" % command
+            raise YumError, "Error performing yum command: %s" % command
         
         return (retval, output)
         
@@ -235,11 +290,8 @@ class Root:
 
         cmd = "%s -c 'rpm -Uvh --nodeps %s' %s" % (self.config['runuser'], 
                           rootdest, self.config['chrootuser'])
-        self.root_log(cmd)
         (retval, output) = self.do_chroot(cmd)
         
-        self.root_log(output)
-        
         if retval != 0:
             msg = "Error installing srpm: %s" % srpmfn
             self.root_log(msg)
@@ -261,7 +313,6 @@ class Root:
                     self.target_arch, chrootspec, self.config['chrootuser'])
         
         (retval, output) = self.do_chroot(cmd)
-        self.root_log(output)
         if retval != 0:
             raise PkgError, "Error building srpm from installed spec. See Root log."
             
@@ -278,7 +329,7 @@ class Root:
         hdr = rpmUtils.miscutils.hdrFromPackage(ts, srpm)
         
         # get text buildreqs
-        buildreqs = self._text_requires_from_hdr(hdr)
+        buildreqs = self._text_requires_from_hdr(hdr, srpm)
         arg_string = ""
         for item in buildreqs:
             
@@ -288,7 +339,6 @@ class Root:
         # pass build reqs (as strings) to installer
         if arg_string != "":
             (retval, output) = self.yum('resolvedep %s' % arg_string)
-            self.root_log(output)
             for line in output.split('\n'):
                 if line.find('No Package Found for') != -1:
                     errorpkg = line.replace('No Package Found for', '')
@@ -311,17 +361,14 @@ class Root:
         srpmfn = os.path.basename(srpm_in)
         # run with --nodeps b/c of the check above we know we have our build
         # deps satisfied.
-        cmd = "%s -c 'rpmbuild --rebuild  --target %s --nodeps %s' %s" % (
+        cmd = "cd /;%s -c 'rpmbuild --rebuild  --target %s --nodeps %s' %s" % (
              self.config['runuser'], self.target_arch, srpm_in, 
              self.config['chrootuser'])
         
         self.state("build")
 
-        self.root_log(cmd)
         (retval, output) = self.do_chroot(cmd)
         
-        self.build_log(output)
-        
         if retval != 0:
             raise BuildError, "Error building package from %s, See build log" % srpmfn
         
@@ -379,7 +426,6 @@ class Root:
         
         if retval != 0:
             if output.find('already mounted') == -1: # probably won't work in other LOCALES
-                self.root_log(output)
                 error("could not mount proc error was: %s" % output)
         
         # devpts
@@ -395,7 +441,6 @@ class Root:
 
         if retval != 0:
             if output.find('already mounted') == -1: # probably won't work in other LOCALES
-                self.root_log(output)
                 raise RootError, "could not mount /dev/pts error was: %s" % output
         
 
@@ -407,7 +452,6 @@ class Root:
     
         if retval != 0:
             if output.find('not mounted') == -1: # this probably won't work in other LOCALES
-                self.root_log(output)
                 raise RootError, "could not umount %s error was: %s" % (path, output)
 
     
@@ -436,9 +480,31 @@ class Root:
     def do(self, command):
         """execute given command outside of chroot"""
         
-        self.debug("Executing %s" % command)
-        (status, output) = commands.getstatusoutput(command)
+        retval = 0
+        msg = "Executing %s" % command
+        self.debug(msg)
+        self.root_log(msg)
 
+        if hasattr(self, '_root_log'):
+            logfile = self._root_log
+        else:
+            logfile = self.tmplog
+        if self.state() == "build":
+            logfile = self._build_log
+
+        pipe = os.popen('{ ' + command + '; } 2>&1', 'r')
+        output = ""
+        for line in pipe:
+            logfile.write(line)
+            if self.config['debug']:
+                print line[:-1]
+                sys.stdout.flush()
+            logfile.flush()
+            output += line
+        status = pipe.close()
+        if status is None:
+            status = 0
+        
         if os.WIFEXITED(status):
             retval = os.WEXITSTATUS(status)
 
@@ -469,7 +535,7 @@ class Root:
         
         return (ret, output)
 
-    def _text_requires_from_hdr(self, hdr):
+    def _text_requires_from_hdr(self, hdr, srpm):
         """take a header and hand back a unique'd list of the requires as
            strings"""
            
@@ -487,6 +553,23 @@ class Root:
             req = rpmUtils.miscutils.formatRequire(n, v, f)
             reqlist.append(req)
         
+        # Extract SRPM name components - still not nice, shouldn't this
+        # be somewhere in the "hdr" parameter?
+        fname = os.path.split(str(srpm))[1]
+        name, ver, rel, epoch, arch = rpmUtils.miscutils.splitFilename(fname)
+
+        # Add the 'more_buildreqs' for this SRPM (if defined)
+        for this_srpm in ['-'.join([name,ver,rel]),
+                          '-'.join([name,ver]),
+                          '-'.join([name]),]:
+            if self.config['more_buildreqs'].has_key(this_srpm):
+                more_reqs = self.config['more_buildreqs'][this_srpm]
+                if type(more_reqs) in (type(u''), type(''),):
+                    more_reqs = [more_reqs] # be nice if we get a string
+                for req in more_reqs:
+                    reqlist.append(req)
+                break
+        
         return rpmUtils.miscutils.unique(reqlist)
     
     def _prep_install(self):
@@ -497,6 +580,7 @@ class Root:
         for item in [self.basedir, self.rootdir, self.statedir, self.resultdir,
                      os.path.join(self.rootdir, 'var/lib/rpm'),
                      os.path.join(self.rootdir, 'var/log'),
+                     os.path.join(self.rootdir, 'var/lock/rpm'),
                      os.path.join(self.rootdir, 'dev'),
                      os.path.join(self.rootdir, 'etc/rpm'),
                      os.path.join(self.rootdir, 'tmp'),
@@ -522,7 +606,6 @@ class Root:
             if not os.path.exists(devpath):
                 (retval, output) = self.do(cmd)
                 if retval != 0:
-                    self.root_log(output)
                     raise RootError, "could not mknod error was: %s" % output
 
         # link fd to ../proc/self/fd
@@ -530,13 +613,26 @@ class Root:
         if not os.path.exists(devpath):
             os.symlink('../proc/self/fd', devpath)
         
+        fd = 0
+        for item in ('stdin', 'stdout', 'stderr'):
+            devpath =  os.path.join(self.rootdir, 'dev', item)
+            if not os.path.exists(devpath):
+                fdpath = os.path.join('../proc/self/fd', str(fd))
+                os.symlink(fdpath, devpath)
+            fd += 1
+
         for item in [os.path.join(self.rootdir, 'etc', 'mtab'),
                      os.path.join(self.rootdir, 'etc', 'fstab'),
                      os.path.join(self.rootdir, 'var', 'log', 'yum.log')]:
-            fo = open(item, 'w')
-            fo.close()
+            if not os.path.exists(item):
+                fo = open(item, 'w')
+                fo.close()
         
         # write in yum.conf into chroot
+        if os.path.exists( os.path.join(self.rootdir, 'etc', 'yum.conf')):
+            cmd = "chown %s.%s /etc/yum.conf" % (self.config['chrootuid'],
+                self.config['chrootgid'])
+            self.do_chroot(cmd, fatal = True)
         yumconf = os.path.join(self.rootdir, 'etc', 'yum.conf')
         yumconf_fo = open(yumconf, 'w')
         yumconf_content = self.config['yum.conf']
@@ -545,18 +641,53 @@ class Root:
         # files in /etc that need doing
         filedict = self.config['files']
         for key in filedict:
+            # ensure permisssions
+            if os.path.exists( "%s%s" % (self.rootdir, key) ):
+                cmd = "chown %s.%s %s" % (self.config['chrootuid'],
+                    self.config['chrootgid'], "%s" % key)
+                self.do_chroot(cmd, fatal = True)
+
+            # write file
             fn = '%s%s' % (self.rootdir, key)
             fo = open(fn, 'w')
             fo.write(filedict[key])
             fo.close()
 
     def _make_our_user(self):
+        if not os.path.exists(os.path.join(self.rootdir, 'usr/sbin/useradd')):
+            raise RootError, "Could not find useradd in chroot, maybe the install failed?"
         # should check if the user exists first
         # make the buildusers/groups
+        need_add_user = 0
         if not os.path.exists(self.rootdir + self.homedir):
-            if not os.path.exists(os.path.join(self.rootdir, 'usr/sbin/useradd')):
-                raise RootError, "Could not find useradd in chroot, maybe the install failed?"
-            cmd = '/usr/sbin/useradd -u %s -d %s %s' % (self.config['chrootuid'], 
+            need_add_user = 1
+        else:
+            # check for the following conditions:
+            #  -- using cache and current user is different from original cache creator
+            #  -- using --no-clean and current user is different from original creator
+            curruid = self.config['chrootuid']
+            chrootuid = None
+            passwd = os.path.join(self.rootdir, 'etc', 'passwd')
+
+            # find UID used to set up buildroot
+            fd = open( passwd, "r" )
+            while 1:
+                line = fd.readline()
+                if line == "": break
+                if line.startswith(self.config["chrootuser"]): 
+                    chrootuid = int(line.split(":")[2])
+
+            # do fixups if they are different
+            # if uid is different, assume we need to fix gid also
+            if chrootuid is not None and curruid != chrootuid:
+                need_add_user = 1
+                self.do_chroot('/usr/sbin/userdel -r %s' % self.config["chrootuser"], fatal = False)
+                self.do_chroot('/usr/sbin/groupdel %s' % self.config["chrootgroup"], fatal = False)
+                self.do_chroot('chown -R %s.%s %s' % (self.config["chrootuid"], self.config["chrootgid"], self.config["chroothome"]), fatal = False)
+                # may need a few other chown here if there are other files that have to be edited
+
+        if need_add_user:
+            cmd = '/usr/sbin/useradd -m -u %s -d %s %s' % (self.config['chrootuid'], 
                     self.homedir, self.config['chrootuser'])
             self.do_chroot(cmd, fatal = True)
 
@@ -571,14 +702,17 @@ class Root:
         for subdir in ('RPMS', 'SRPMS', 'SOURCES', 'SPECS', 'BUILD', 'originals'):
             cmd = "mkdir -p %s/%s" % (self.builddir, subdir)
             self.do_chroot(cmd, fatal = True)
-            cmd = "chown %s.%s %s/%s" % (self.config['chrootuser'], 
-               self.config['chrootgroup'], self.builddir, subdir)
-            self.do_chroot(cmd, fatal = True)
+
+        # change ownership so we can write to build home dir
+        cmd = "chown -R %s.%s %s" % (self.config['chrootuser'], 
+           self.config['chrootgroup'], self.homedir)
+        self.do_chroot(cmd, fatal = True)
         
         # rpmmacros default
         macrofile_out = '%s%s/.rpmmacros' % (self.rootdir, self.homedir)
         if not os.path.exists(macrofile_out):
             rpmmacros = open(macrofile_out, 'w')
+            self.config['macros'] = self.config['macros'] + "\n%%_rpmlock_path	%s/var/lib/rpm/__db.000" % self.basedir
             rpmmacros.write(self.config['macros'])
             rpmmacros.close()
         
@@ -589,41 +723,179 @@ class Root:
         self._build_dir_setup()
         self._mount() # check it again
         
-        # FIXME - do we need this still?
-        # create /boot/kernel.h with a warning
-        #self.do_chroot ("mkdir -p /boot", fatal = True)
-        #self.do_chroot ("echo '#ifndef __BOOT_KERNEL_H_' > /boot/kernel.h", fatal = True)
-        #self.do_chroot ("echo '#define __BOOT_KERNEL_H_' >> /boot/kernel.h", fatal = True)
-        #self.do_chroot ("echo '#error This is a kernel.h generated by mach, including this indicates a build error !' >> /boot/kernel.h", fatal = True)
-        #self.do_chroot ("echo '#endif /* __BOOT_KERNEL_H_ */' >> /boot/kernel.h", fatal = True)
-        
 def command_parse():
     """return options and args from parsing the command line"""
     
     usage = """
-    usage: mock [options] /path/to/srpm
-    optional commands: 
+    usage:
+           mock [options] [rebuild] /path/to/srpm(s)
+           mock [options] chroot <cmd>
+           mock [options] {init|clean|shell}
+    commands: 
+        rebuild - build the specified SRPM(s) [default command]
+        chroot - run the specified command within the chroot
+        shell - run an interactive shell within specified chroot
         clean - clean out the specified chroot
         init - initialize the chroot, do not build anything"""
     parser = OptionParser(usage=usage, version=__VERSION__)
     parser.add_option("-r", action="store", type="string", dest="chroot",
-                      default='default', 
-                      help="chroot name/config file name default: %default")
-    parser.add_option("--no-clean", action ="store_true", dest="dirty", 
-             help="do not clean chroot before building")
+                      help="chroot name/config file name default: %default", 
+                      default='default')
+    parser.add_option("--no-clean", action ="store_false", dest="clean", 
+                      help="do not clean chroot before building", default=True)
     parser.add_option("--arch", action ="store", dest="arch", 
-             default=None, help="target build arch")
+                      default=None, help="target build arch")
     parser.add_option("--debug", action ="store_true", dest="debug", 
-             default=False, help="Output copious debugging information")
+                      default=False, help="Output copious debugging information")
     parser.add_option("--resultdir", action="store", type="string", 
-             default=None, help="path for resulting files to be put")
+                      default=None, help="path for resulting files to be put")
     parser.add_option("--statedir", action="store", type="string", default=None,
-            help="path for state dirresulting files to be put")
+                      help="Path to directory where state information is written")
     parser.add_option("--uniqueext", action="store", type="string", default=None,
-            help="Arbitrary, unique extension to append to buildroot directory name")
-
-    return parser.parse_args()
+                      help="Arbitrary, unique extension to append to buildroot directory name")
+    parser.add_option("--configdir", action="store", dest="configdir", default=None,
+                      help="Change where config files are found")
+    parser.add_option("--quiet", action ="store_true", dest="quiet", 
+                      default=False, help="quiet down output")
+    parser.add_option("--autocache", action ="store_true", dest="use_cache",
+                      default=False, help="Turn on build-root caching")
+    parser.add_option("--rebuildcache", action ="store_true", dest="rebuild_cache",
+                      default=False, help="Force rebuild of build-root cache")
     
+    return parser.parse_args()
+
+def setup_default_config_opts(config_opts):
+    config_opts['basedir'] = '/var/lib/mock/' # root name is automatically added to this
+    config_opts['chroot'] = '/usr/sbin/mock-helper chroot'
+    config_opts['mount'] = '/usr/sbin/mock-helper mount'
+    config_opts['umount'] = '/usr/sbin/mock-helper umount'
+    config_opts['rm'] = '/usr/sbin/mock-helper rm'
+    config_opts['mknod'] = '/usr/sbin/mock-helper mknod'
+    config_opts['yum'] = '/usr/sbin/mock-helper yum'
+    config_opts['runuser'] = '/sbin/runuser'
+    config_opts['chroot_setup_cmd'] = 'install buildsys-build'
+    config_opts['chrootuser'] = 'mockbuild'
+    config_opts['chrootgroup'] = 'mockbuild'
+    config_opts['chrootuid'] = 500
+    config_opts['chrootgid'] = 500
+    config_opts['chroothome'] = '/builddir'
+    config_opts['clean'] = True
+    config_opts['debug'] = False
+    config_opts['quiet'] = False
+    config_opts['target_arch'] = 'i386'
+    config_opts['files'] = {}
+    config_opts['yum.conf'] = ''
+    config_opts['macros'] = """
+%%_topdir %s/build
+%%_rpmfilename   %%%%{NAME}-%%%%{VERSION}-%%%%{RELEASE}.%%%%{ARCH}.rpm
+    
+""" % config_opts['chroothome']
+    
+    config_opts['more_buildreqs'] = {}
+    config_opts['files']['/etc/resolv.conf'] = "nameserver 192.168.1.1\n"
+    config_opts['files']['/etc/hosts'] = "127.0.0.1 localhost localhost.localdomain\n"
+
+    # caching-related config options
+    config_opts['rebuild_cache'] = False
+    config_opts['use_cache'] = False
+    config_opts['pack_cmd'] = "/usr/sbin/mock-helper pack"
+    config_opts['unpack_cmd'] = "/usr/sbin/mock-helper unpack"
+    config_opts['cache_ext'] = ".tar.gz"
+    config_opts['cache_topdir'] = "root-cache"
+    config_opts['max_cache_age_days'] = 15
+
+def set_config_opts_per_cmdline(config_opts, options):
+    # do some other options and stuff
+    if options.arch:
+        config_opts['target_arch'] = options.arch
+    if not options.clean:
+        config_opts['clean'] = options.clean
+    if options.debug:
+        config_opts['debug'] = options.debug
+    if options.quiet:
+        config_opts['quiet'] = options.quiet
+    if options.use_cache:
+        config_opts['use_cache'] = options.use_cache
+    if options.rebuild_cache:
+        config_opts['rebuild_cache'] = options.rebuild_cache
+    if config_opts['rebuild_cache']: 
+        config_opts['use_cache'] = True
+    if config_opts['rebuild_cache']: 
+        config_opts['use_cache'] = True
+    if options.resultdir:
+        config_opts['resultdir'] = options.resultdir
+    if options.statedir:
+        config_opts['statedir'] = options.statedir
+    if options.uniqueext:
+        config_opts['unique-ext'] = options.uniqueext
+
+def do_clean(config_opts, init=0):
+        my = None
+        try:
+            my = Root(config_opts)
+            if init: my.prep()
+        except Error, e:
+            print e
+            if my:
+                my.close()
+            sys.exit(100)
+
+        my.close()
+        if init:
+            print 'Finished initializing root'
+        else:
+            print 'Finished cleaning root'
+
+def do_run_cmd(config_opts, cmd, env='', raw_chroot=0):
+        my = Root(config_opts)
+        my.debug("executing: %s" % cmd)
+        my._mount()
+        if raw_chroot: 
+            cmd = '%s %s %s %s' % (env, config_opts['chroot'], my.rootdir, cmd)
+            os.system(cmd)
+        else:
+            my.do_chroot(cmd, True)
+        my.close()
+        my.debug('finished chroot command')
+               
+def ensure_filetype_srpm(srpms):
+    for srpm in srpms:
+        ts = rpmUtils.transaction.initReadOnlyTransaction()
+        try:
+            hdr = rpmUtils.miscutils.hdrFromPackage(ts, srpm)
+        except rpmUtils.RpmUtilsError, e:
+            error("Specified srpm %s cannot be found/opened" % srpm)
+            sys.exit(50)
+   
+        if hdr[rpm.RPMTAG_SOURCEPACKAGE] != 1:
+            error("Specified srpm isn't a srpm!  Can't go on")
+            sys.exit(50)
+
+def do_rebuild(config_opts, srpms):
+    # Prep build root
+    my = None  # if Root() fails, my will be undefined so we force it to None
+    try:
+        my = Root(config_opts)
+        os.umask(0022) # set a umask- protects from paranoid whackjobs with an 002 umask
+    except Error, e:
+        error(e)
+        if my:
+            my.close()
+        sys.exit(e.resultcode)
+   
+    for srpm in srpms:
+        try:
+            my.prep()
+            my.build(srpm)
+        except Error, e:
+            error(e)
+            if my:
+                my.close()
+            sys.exit(e.resultcode)
+    
+    my.close()
+    print "Results and/or logs in: %s" % my.resultdir
+
 def main():
     # before we go on, make sure the user is a member of the 'mock' group.
     member = False
@@ -644,38 +916,9 @@ def main():
         error("Don't try to run mock as root!")
         sys.exit(1)
         
-    # config path
-    config_path='/etc/mock'
-    
     # defaults
     config_opts = {}
-    config_opts['basedir'] = '/var/lib/mock/' # root name is automatically added to this
-    config_opts['chroot'] = '/usr/sbin/mock-helper chroot'
-    config_opts['mount'] = '/usr/sbin/mock-helper mount'
-    config_opts['umount'] = '/usr/sbin/mock-helper umount'
-    config_opts['rm'] = '/usr/sbin/mock-helper rm'
-    config_opts['mknod'] = '/usr/sbin/mock-helper mknod'
-    config_opts['yum'] = '/usr/sbin/mock-helper yum'
-    config_opts['runuser'] = '/sbin/runuser'
-    config_opts['buildgroup'] = 'build'
-    config_opts['chrootuser'] = 'mockbuild'
-    config_opts['chrootgroup'] = 'mockbuild'
-    config_opts['chrootuid'] = 500
-    config_opts['chrootgid'] = 500
-    config_opts['chroothome'] = '/builddir'
-    config_opts['clean'] = True
-    config_opts['debug'] = False
-    config_opts['target_arch'] = 'i386'
-    config_opts['files'] = {}
-    config_opts['yum.conf'] = ''
-    config_opts['macros'] = """
-%%_topdir %s/build
-%%_rpmfilename   %%%%{NAME}-%%%%{VERSION}-%%%%{RELEASE}.%%%%{ARCH}.rpm
-    
-""" % config_opts['chroothome']
-    
-    config_opts['files']['/etc/resolv.conf'] = "nameserver 192.168.1.1\n"
-    config_opts['files']['/etc/hosts'] = "127.0.0.1 localhost localhost.localdomain\n"
+    setup_default_config_opts(config_opts)
     
     # cli option parsing
     (options, args) = command_parse()
@@ -683,6 +926,20 @@ def main():
     if len(args) < 1:
         error("No srpm or command specified - nothing to do")
         sys.exit(50)
+
+    # config path -- can be overridden on cmdline
+    config_path='/etc/mock'
+    if options.configdir:
+        config_path = options.configdir
+    
+    # Read in the default values which can be overwritten
+    # with the more specific config being loaded below.
+    cfg = os.path.join(config_path, 'defaults.cfg')
+    if os.path.exists(cfg):
+        execfile(cfg)
+    else:
+        if config_path != "/etc/mock" and os.file.exists("/etc/mock/defaults.cfg"):
+            execfile("/etc/mock/defaults.cfg")
     
     # read in the config file by chroot name
     if options.chroot.endswith('.cfg'):
@@ -696,94 +953,45 @@ def main():
         error("Could not find config file %s for chroot %s" % (cfg, options.chroot))
         sys.exit(1)
     
-    # do some other options and stuff
-    if options.arch:
-        config_opts['target_arch'] = options.arch
+    # cmdline options override config options
+    set_config_opts_per_cmdline(config_opts, options)
     
-    if options.dirty:
-        config_opts['clean'] = False
-    else:
-        config_opts['clean'] = True
-        
-    config_opts['debug'] = options.debug
-    
-    if options.resultdir:
-        config_opts['resultdir'] = options.resultdir
-
-    if options.statedir:
-        config_opts['statedir'] = options.statedir
-
-    if options.uniqueext:
-        config_opts['unique-ext'] = options.uniqueext
-
     # do whatever we're here to do
     if args[0] == 'clean':
         # unset a --no-clean
         config_opts['clean'] = True
-        try:
-            my = None
-            my = Root(config_opts)
-        except Error, e:
-            print e
-            if my:
-                my.close()
-            sys.exit(100)
-
-        my.close()
-        print 'Finished cleaning root'
+        do_clean(config_opts, init=0)
         
     elif args[0] == 'init':
-        try:
-            my = None
-            my = Root(config_opts)
-            my.prep()
-        except Error, e:
-            print e
-            if my:
-                my.close()
-            sys.exit(100)
+        do_clean(config_opts, init=1)
 
-        my.close()
-        print 'Finished initializing root'
+    elif args[0] == 'chroot':
+        # catch-all for executing arbitrary commands in the chroot
+        config_opts['clean'] = config_opts['quiet'] = False
+        cmd = ' '.join(args[1:])
+        do_run_cmd(config_opts, cmd, raw_chroot=0)
         
+    elif args[0] == 'shell':
+        # debugging tool for interactive poking around in the chroot
+        config_opts['clean'] = config_opts['quiet'] = False
+        do_run_cmd(config_opts, "/bin/bash", env='PS1="mock-chroot> "', raw_chroot=1)
+
     else:
         if args[0] == 'rebuild':
             if len(args) > 1:
-                srpm = args[1]
+                srpms = args[1:]
             else:
                 error("No package specified to rebuild command.")
                 sys.exit(50)
         else:
-            srpm = args[0]
+            srpms = args[0:]
 
-        ts = rpmUtils.transaction.initReadOnlyTransaction()
-        try:
-            hdr = rpmUtils.miscutils.hdrFromPackage(ts, srpm)
-        except rpmUtils.RpmUtilsError, e:
-            error("Specified srpm %s cannot be found/opened" % srpm)
-            sys.exit(50)
-    
-        if hdr[rpm.RPMTAG_SOURCEPACKAGE] != 1:
-            error("Specified srpm isn't a srpm!  Can't go on")
-            sys.exit(50)
-    
-        try:
-            my = None  # if Root() fails, my will be undefined so we force it to None
-            my = Root(config_opts)
-            my.prep()
-            my.build(srpm)
-        except Error, e:
-            error(e)
-            if my:
-                my.close()
-            sys.exit(e.resultcode)
-    
-        my.close()
-        print "Results and/or logs in: %s" % my.resultdir
+        # exit here if everything isn't kosher
+        ensure_filetype_srpm(srpms)
 
+        do_rebuild(config_opts, srpms)
 
 if __name__ == '__main__':
     main()
 
 
-
